﻿package {
	import flash.display.Graphics;
	import flash.display.Shape;
	import flash.display.Sprite;
	import flash.events.Event;
	import flash.events.MouseEvent;
	
	[SWF(width = "640", height = "480", frameRate="30", backgroundColor = "#000000")]
	
	/**
	 * ...
	 * @author Aaron Steed, robotacid.com
	 */
	public class Main extends Sprite {
		
		public var map:Array;
		
		public var start_x:int = 0;
		public var start_y:int = 0;
		
		public var map_shape:Shape;
		public var lightning_shape:Shape;
		
		public static const MAP_WIDTH:int = 80;
		public static const MAP_HEIGHT:int = 60;
		public static const SCALE:Number = 4;
		public static const INV_SCALE:Number = 1.0 / 4;
		public static const WIDTH:Number = 320;
		public static const HEIGHT:Number = 240;
		
		public var nodes:Array/*Node*/ = [];
		public var pos:Array/*Pixel*/ = [
			new Pixel( -1, -1),
			new Pixel( 0, -1),
			new Pixel( 1, -1),
			new Pixel( -1, 0),
			new Pixel( 1, 0),
			new Pixel( -1, 1),
			new Pixel( 0, 1),
			new Pixel( 1, 1)
		];
		
		public function Main():void {
			if (stage) init();
			else addEventListener(Event.ADDED_TO_STAGE, init);
		}
		
		private function init(e:Event = null):void {
			removeEventListener(Event.ADDED_TO_STAGE, init);
			// entry point
			scaleX = scaleY = 2;
			for(var i:int = 0; i < pos.length; i++){
				nodes.push(new Node());
			}
			lightning_shape = new Shape();
			map_shape = new Shape();
			addChild(map_shape);
			addChild(lightning_shape);
			map = [];
			var r:int, c:int;
			for(r = 0; r < MAP_HEIGHT; r++){
				map[r] = [];
				for(c = 0; c < MAP_WIDTH; c++){
					map_shape.graphics.lineStyle(1, 0x333333);
					map[r][c] = Math.random() < 0.1 ? 1 : 0;
					map_shape.graphics.beginFill(map[r][c] ? 0x999999 : 0x000000);
					map_shape.graphics.drawRect(c * SCALE, r * SCALE, SCALE, SCALE);
				}
			}
			stage.addEventListener(MouseEvent.CLICK, click);
			addEventListener(Event.ENTER_FRAME, loop);
		}
		
		private function click(e:MouseEvent):void{
			start_x = mouseX * INV_SCALE;
			start_y = mouseY * INV_SCALE;
		}
		
		private function loop(e:Event = null):void{
			lightning_shape.graphics.clear();
			drawLightning(lightning_shape.graphics, start_x, start_y);
		}
		
		private function drawLightning(gfx:Graphics, x:int, y:int):void{
			
			var finish_x:int = mouseX * INV_SCALE;
			var finish_y:int = mouseY * INV_SCALE;
			
			while(x != finish_x || y != finish_y){
				var node:Node = getNextNode(x, y, finish_x, finish_y);
				gfx.lineStyle(1, 0xFFFFFF);
				gfx.moveTo((SCALE * 0.5) + x * SCALE, (SCALE * 0.5) + y * SCALE);
				var dx:int = node.x - x;
				var dy:int = node.y - y;
				dx += (Math.random() * -dy * 1.5) + (Math.random() * dy * 1.5);
				dy += (Math.random() * -dx * 1.5) + (Math.random() * dx * 1.5);
				x += dx;
				y += dy;
				if(x < 0 || y < 0 || x > MAP_WIDTH-1 || y > MAP_HEIGHT-1 || map[y][x]) break;
				gfx.lineTo((SCALE * 0.5) + x * SCALE, (SCALE * 0.5) + y * SCALE);
				lightningBranch(gfx, x, y, dx, dy, Math.random());
				
			}
			
		}
		
		private function lightningBranch(gfx:Graphics, x:int, y:int, dx:int, dy:int, decay:Number):void{
			gfx.lineStyle(1, 0xFFFFFF, decay);
			gfx.moveTo((SCALE * 0.5) + x * SCALE, (SCALE * 0.5) + y * SCALE);
			x += dx;
			y += dy;
			if(x < 0 || y < 0 || x > MAP_WIDTH-1 || y > MAP_HEIGHT-1 || map[y][x]) return;
			gfx.lineTo((SCALE * 0.5) + x * SCALE, (SCALE * 0.5) + y * SCALE);
			if(Math.random() < decay){
				dx += (Math.random() * -dy * 1.5) + (Math.random() * dy * 1.5);
				dy += (Math.random() * -dx * 1.5) + (Math.random() * dx * 1.5);
				lightningBranch(gfx, x, y, dx, dy, decay * Math.random());
			}
			
		}
		
		private function getNextNode(x:int, y:int, target_x:int, target_y:int):Node{
			var i:int, best:Number = Number.MAX_VALUE, choice:int = -1;
			for(i = 0; i < nodes.length; i++){
				nodes[i].x = x + pos[i].x;
				nodes[i].y = y + pos[i].y;
				nodes[i].dist(target_x, target_y);
				if(nodes[i].h < best){
					best = nodes[i].h;
					choice = i;
				}
			}
			return nodes[choice];
		}
		
		
	}
	
}

class Pixel {
		
	public var x:int;
	public var y:int;
	
	public function Pixel(x:int = 0, y:int = 0) {
		this.x = x;
		this.y = y;
	}
	/* Manhattan distance */
	public function mDist(p:Pixel):Number{
		return Math.abs(p.x - x) + Math.abs(p.y - y);
	}
	public function toString():String {
		return "(" + x + "," + y + ")";
	}
	public function copy():Pixel{
		return new Pixel(x, y);
	}
	
}

class Node extends Pixel{
	
	public var walkable:Boolean;
	public var h:Number;
	
	public function Node(x:int = 0, y:int = 0) {
		super(x, y);
		
	}
	
	public function dist(x:int, y:int):void{
		var dx:int = this.x - x;
		var dy:int = this.y - y;
		h = Math.sqrt(dx * dx + dy * dy);
	}
	
}